# pio_buzzer_2.py - 12.09.2021
#
# siehe auch pio_buzzer_1.py
# Die Kommunikation zwischen dem Taktgeber und dem Tongenerator
# erfolgt über den irq(4).
# Die Ein-Auszeiten werden durch ein 32-Bit-Pattern bestimmt,
# das über einen Zeitraum von ca 1 Sekunde abläuft.
# default = 0x0000_FFFF -> 1/2 Sek. Beep, 1/2 Sek. Pause
#           0x00FF_00FF -> 1/4 Sek. Beep, 1/4 Sel. Pause  ...
# Das höchstwertige Bit soll 0 sein, damit am Summer am Ende
# keine Spannung mehr anliegt.

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

class Buzzer(object):
    def __init__(self, output_pin, buzzer_freq=3250):
        self.sm0 = StateMachine(0, self.buzzer, freq=buzzer_freq*2, sideset_base=Pin(output_pin))
        self.sm1 = StateMachine(1, self.cycle, freq=2000)
        self.sm0.active(1)
        self.sm1.active(1)
        
    def beep(self, cnt=1, pattern=0x0000_FFFF):
        # eine Pause einlegen
        if cnt < 1:
            self.sm1.put(0)
        else:
            # das höchstwerige Bit löschen
            pattern &= 0x7FFF_FFFF
            for _ in range (cnt):
                self.sm1.put(pattern)
    
    def tx_free(self):
        # Anzahl der freien Words im RX-Fifo
        return 8 - self.sm1.tx_fifo()
    
    @staticmethod
    @asm_pio(sideset_init=PIO.OUT_LOW)
    # Hier wird der Ton generiert = 1/2 des F_PIO-Taktes
    def buzzer():
        wrap_target()
        
        nop()            .side(1)  #  1
        wait(0, irq, 4)  .side(0)  #  2
        wrap()
        
    @staticmethod
    @asm_pio(out_shiftdir=PIO.SHIFT_RIGHT, fifo_join=PIO.JOIN_TX)
    # hier werden Pulse und Pause generiert,
    # die den Buzzer ein- und ausschalten.
    def cycle():
        wrap_target()
        pull(block)                 #  1 blockend warten auf eine Eingabe
        
        label("next")               #  2 
        out(x, 1)  [29]             #  3 1 Bit nach x ausgeben
        
        jmp(not_x, "off")           #  4 das Bit prüfen und springen
        irq(clear, 4)               #  5 Einschalten
        jmp("end")                  #  6 
        
        label("off")
        irq(4)                      #  7 Ausschalten
        
        label("end")
        jmp(not_osre, "next") [31]  #  8 alle 32 Bit rausshiften
        
        wrap()                      #    dann das nächste Word holen


if __name__ == "__main__":
    
    buzzer_pin = 19
    buzz = Buzzer(output_pin=buzzer_pin)

    buzz.beep()  # Standard-Beep
    buzz.beep(0) # Pause
    buzz.beep(1, 0x3333_3333)
    buzz.beep(1, 0x1248_00FF)
    buzz.beep(1, 0x1248_1248)
    
    print("\ndone")
