# pio_counter_64.py - 12.09.2021
#
# Ein Timer/Counter zählt die Zeit zwischen Rising und Falling Edge
# Die Pause zwischen den Pulsen wird ignoriert.
# Der erwartete Ruhepegel ist low.
# zählt bis 2 ** 32 - 1

from machine import Pin
from rp2 import PIO, StateMachine, asm_pio
import time
import uarray as arr

# ----------------------------------------------------------------------------
@rp2.asm_pio(
        #sideset_init=rp2.PIO.OUT_HIGH, 
        in_shiftdir=rp2.PIO.SHIFT_RIGHT,
        fifo_join=rp2.PIO.JOIN_RX ) 

def pulse_count():   
    wrap_target()
    mov(x, invert(null))           #  1 Zähler 0xFFFF_FFFF nach x
    wait(1, pin, 0)                #  2 auf steigende Flanke warten
        
    label("next") 
    jmp(x_dec, "next1")            #  3 decrementieren
    jmp("exit")                    #  4 Überlauf
    
    label("next1")
    jmp(pin, "next")               #  5 den Pin testen, wenn high, dann beenden
                                   #    weiter decrementieren
    label("exit")

    in_(x, 32)                     #  6 der Pin ist low, den Zähler ins Register isr
    push()                         #  7 ins Fifo
    irq(rel(0))                    #  8 Interrupt auslösen

    wrap()                 
    
class Pulse_Counter(object):
    def __init__(self, statemachine, rx_pin, buffer_size=64, freq=120_000_0):
        self.sm = StateMachine(statemachine, pulse_count, 
                in_base=Pin(rx_pin, Pin.IN, Pin.PULL_UP),
                jmp_pin=Pin(rx_pin, Pin.IN, Pin.PULL_UP),
                freq=freq)
        
        self.sm.restart()
        self.sm.irq(self.irq_handler)
        self.rx_buffer = arr.array("I", [0] * buffer_size)
        self.rx_cnt = 0

    def activate(self, state=1):
        self.sm.active(1 if state else 0)
                   
    def irq_handler(self, _):
        self.rx_buffer[self.rx_cnt] = self.sm.get()
        self.rx_cnt += 1
        
    def get_data(self):
        return (self.rx_cnt, self.rx_buffer)
    
if __name__ == "__main__":
    
    # -- Einstellung ------------------------
    
    rx_pin = 17       
    F_PIO = 2_000_000
    
    # ----------------------------------------------

    def print_rx_buffer():
        for i in range(rx_cnt):
            x = rx_buffer[i]
            #print("tx: {:8}. rx: {:10}".format(test_pulse[i], (256**4 - 1)  - x))
            print("# {:02} {:8}".format(i, (0xFFFF_FFFF)  - x))
    
    rx = Pulse_Counter(statemachine=4, rx_pin=rx_pin, freq=F_PIO)
    rx.activate()
    
    print("Ctrl-C to stop waiting + print !")
    print("\nF_PIO: {} Hz, Resolution: {} Nanosec.".format(F_PIO, 2 * 10**9 / F_PIO))
    
    try:
        time.sleep(5)
    except KeyboardInterrupt:
        pass
    
    rx_cnt, rx_buffer = rx.get_data()
            
    if rx_cnt:        
        print_rx_buffer()
    else:
        print("nothing received")

    rx.activate(0)
    print("\ndone")
    