# pio_ir_nec_tx.py  -  12.09.2021

import time
from rp2 import PIO, StateMachine, asm_pio
from machine import Pin

# IR-Remote-Transmitter für das NEC-Protikoll
#
# Es werden 32 Datenbit übertragen
# Das zweit-höchstwertige Byte [2] wird als Datenbyte interpretiert,
# Das höchstwertige Byte [3] ist das invertierte Datenbyte.
# das niedrigwertigste Byte [0] als Adressbyte.
# das zweitniedrigste Byte [1] ist das invertierte Adressbyte,
# 
# 32 Bit Daten  Byte[3]     Byte[2]    Byte[1]    Byte[0]
#               inv. Daten  Daten      inv. Adr.  Adr
# -----------------------------------------

@asm_pio(sideset_init=PIO.OUT_LOW)
# Hier wird der 38KHz Takt für die IR-Led generiert
def ir_led():
    wrap_target()
        
    nop()            .side(1)  #  1
    wait(0, irq, 4)  .side(0)  #  2
    
    wrap()

# ---------------------------------------

@asm_pio(out_shiftdir=PIO.SHIFT_RIGHT,
             fifo_join=PIO.JOIN_TX)

# Hier werden das Startbit, die Datenbits und das Stopbit moduliert,
# indem per IRQ 4 die IR-Diode ein- und ausgeschaltet wird.
# TX_FIFO ist um RX_FIFO erweitert (auf 8 Worte)
# Träger  : 38 KHz
# Takt    : 80us / 12.500 Hz
# Startbit: 8000 us (100 * 80us)  + 4500 us (56 * 80us)
# Bit_0   = 560 + 560 us   7 * 80 us =  560 us
# Bit_1   = 560 + 1690 us  (7 + 14) * 80 us = 1680 us
# Stop    = 560 us + 30ms Wartezeit
# [Wiederholung: 8000us + 2250 us + 560 us + 100 ms Pause]

def ir_nec():
    wrap_target()              #    
    pull()                     #  1 Die Zeiten für das Startbit werden
                               #    biem ersten pull() übergeben                    
    out(x, 8)                  #  2 98 * 80 us = 8000us 

    irq(clear, 4)              #  3 Träger ein für Startbit 
                               #    hier starte die Zeit
 
    label("delay_1")           #    
    jmp(x_dec, "delay_1")      #  4 irq + (1 + 110) Takte je 80 us = 112 `80 = 8960 us

    irq(4)                     #  5 Träger aus fpr Startbit
    
    out(x, 8)                  #  6 
    label("delay_2")           
    jmp(x_dec, "delay_2")      #  7 (irq + out + pull + (1 + 52)) * 80 = 56 * 80 = 4480us
    
    pull()                     #  8  die zu übermittelnden Daten holen,
    
    label("next_bit")
    irq(clear, 4)              #  9 Ende der Pause, Träger ein
    out(x, 1)              [5] # 10 das nächste Bit nach x
                               #    irq + (out + 5) = 7 * 80 = 560 us
    irq(4)                 [4] # 11 Träger aus für
                               #    (irq +  5) + jmp + jmp = 7 * 80 us = 560 us

    jmp(not_x,"no_nops")       # 12 wenn 0, dann die mächsten nops auslassen
    
    nop()                 [13] # 13 Pause verlängern 1690 - 560 = 1130 (nop + 13) * 80
    #nop() [6]                 #    um 1 sideset nutzen zu können, darf das delay 
    #nop() [6]                 #    nicht grü0er als 7 sein
    
    label("no_nops")
    jmp(not_osre, "next_bit")  # 14 weiter solange wie Bits vorhanden sind

    irq(clear, 4)          [6] # 15 Träger ein für 560 us (irq + 6) = 7 * 80

    irq(4)                 [7] # 16 Träger aus

    set(x, 31)                 # 17 Pause nach einer übertragung einlegen
    label("delay_3")           #    ca. 140ms Abstand zur ggf. nächsten Sognal
                               #    willkürlich gewählt !
    jmp(x_dec, "delay_3") [ 7] # 18 (set + (jmp + 31) * (32)) * 80us > 82ms

    wrap()  
   
# --------------------------------------------------------

class IR_TX(object):
    def __init__(self, ir_pin, freq=38_000):
        self.sm0 = StateMachine(0, ir_led, freq=freq*2, sideset_base=Pin(ir_pin))
        self.sm1 = StateMachine(1, ir_nec, freq=12500)

        self.sm0.active(1)
        self.sm1.active(1)
 
    def send(self, data):      
        msg = (255 - data[1]) * 256 + data[1]                # cmd
        msg = msg * 256**2 + (255 - data[0]) * 256 + data[0] # adr
        
        # Vor jeder Sendung wird vorbereitet:
        # Die Anzahl der Takte für Start_Pulse + Start_Pause in die Statemachine schreiben
        # die werden jeweils 8-bitweise vor dem Startpulse bzw. der Startpause
        # ins X-Register als Counter geladen.
        self.sm1.put(52 * 256 | 110)
        self.sm1.put(msg)
      

if __name__ == "__main__":
    
    ir_tx_pin = 16    # Output-Pin für die IR-LED
 
    adr = 128
    cmd = 12
        
    tx = IR_TX(ir_pin=ir_tx_pin)
    
    tx.send(bytes([adr, cmd]))
    print("Sending -> Adr: {}, Cmd: {}".format(adr, cmd))
   
    time.sleep(.2)

    print("done")
